@@ -39,6 +39,11 @@ then
         exit 0
     else
         echo "code is not formatted correctly! :("
+        echo "Suggested change:"
+        cp -r src clang_format_src
+        $CLANG_FORMAT -style=file -i clang_format_src/*.c clang_format_src/*.h
+        diff -ur src clang_format_src
+        rm -r clang_format_src
         echo "Run '$0 reformat' to fix formatting"
         exit 1
     fi
