@@ -31,7 +31,6 @@ int main(int argc, char **argv) {
     int i;
     int pcre_opts = PCRE_MULTILINE;
     int study_opts = 0;
-    double time_diff;
     worker_t *workers = NULL;
     int workers_len;
     int num_cores;
@@ -40,9 +39,16 @@ int main(int argc, char **argv) {
 
     work_queue = NULL;
     work_queue_tail = NULL;
-    memset(&stats, 0, sizeof(stats));
     root_ignores = init_ignore(NULL, "", 0);
     out_fd = stdout;
+
+    parse_options(argc, argv, &base_paths, &paths);
+    log_debug("PCRE Version: %s", pcre_version());
+    if (opts.stats) {
+        memset(&stats, 0, sizeof(stats));
+        gettimeofday(&(stats.time_start), NULL);
+    }
+
 #ifdef USE_PCRE_JIT
     int has_jit = 0;
     pcre_config(PCRE_CONFIG_JIT, &has_jit);
@@ -51,11 +57,6 @@ int main(int argc, char **argv) {
     }
 #endif
 
-    gettimeofday(&(stats.time_start), NULL);
-
-    parse_options(argc, argv, &base_paths, &paths);
-    log_debug("PCRE Version: %s", pcre_version());
-
 #ifdef _WIN32
     {
         SYSTEM_INFO si;
@@ -86,7 +87,7 @@ int main(int argc, char **argv) {
     if (pthread_mutex_init(&print_mtx, NULL)) {
         die("pthread_mutex_init failed!");
     }
-    if (pthread_mutex_init(&stats_mtx, NULL)) {
+    if (opts.stats && pthread_mutex_init(&stats_mtx, NULL)) {
         die("pthread_mutex_init failed!");
     }
     if (pthread_mutex_init(&work_queue_mtx, NULL)) {
@@ -134,7 +135,7 @@ int main(int argc, char **argv) {
             workers[i].id = i;
             int rv = pthread_create(&(workers[i].thread), NULL, &search_file_worker, &(workers[i].id));
             if (rv != 0) {
-                die("error in pthread_create(): %s", strerror(rv));
+                die("Error in pthread_create(): %s", strerror(rv));
             }
 #if defined(HAVE_PTHREAD_SETAFFINITY_NP) && defined(USE_CPU_SET)
             if (opts.use_thread_affinity) {
@@ -142,10 +143,12 @@ int main(int argc, char **argv) {
                 CPU_ZERO(&cpu_set);
                 CPU_SET(i % num_cores, &cpu_set);
                 rv = pthread_setaffinity_np(workers[i].thread, sizeof(cpu_set), &cpu_set);
-                if (rv != 0) {
-                    die("error in pthread_setaffinity_np(): %s", strerror(rv));
+                if (rv) {
+                    log_err("Error in pthread_setaffinity_np(): %s", strerror(rv));
+                    log_err("Performance may be affected. Use --noaffinity to suppress this message.");
+                } else {
+                    log_debug("Thread %i set to CPU %i", i, i);
                 }
-                log_debug("Thread %i set to CPU %i", i, i);
             } else {
                 log_debug("Thread affinity disabled.");
             }
@@ -182,11 +185,12 @@ int main(int argc, char **argv) {
 
     if (opts.stats) {
         gettimeofday(&(stats.time_end), NULL);
-        time_diff = ((long)stats.time_end.tv_sec * 1000000 + stats.time_end.tv_usec) -
-                    ((long)stats.time_start.tv_sec * 1000000 + stats.time_start.tv_usec);
+        double time_diff = ((long)stats.time_end.tv_sec * 1000000 + stats.time_end.tv_usec) -
+                           ((long)stats.time_start.tv_sec * 1000000 + stats.time_start.tv_usec);
         time_diff /= 1000000;
-
-        printf("%ld matches\n%ld files searched\n%ld bytes searched\n%f seconds\n", stats.total_matches, stats.total_files, stats.total_bytes, time_diff);
+        printf("%ld matches\n%ld files contained matches\n%ld files searched\n%ld bytes searched\n%f seconds\n",
+               stats.total_matches, stats.total_file_matches, stats.total_files, stats.total_bytes, time_diff);
+        pthread_mutex_destroy(&stats_mtx);
     }
 
     if (opts.pager) {
@@ -195,7 +199,6 @@ int main(int argc, char **argv) {
     cleanup_options();
     pthread_cond_destroy(&files_ready);
     pthread_mutex_destroy(&work_queue_mtx);
-    pthread_mutex_destroy(&stats_mtx);
     pthread_mutex_destroy(&print_mtx);
     cleanup_ignore(root_ignores);
     free(workers);
