@@ -18,6 +18,9 @@ int first_file_match = 1;
 const char *color_reset = "\033[0m\033[K";
 
 void print_path(const char *path, const char sep) {
+    if (opts.print_path == PATH_PRINT_NOTHING && !opts.vimgrep) {
+        return;
+    }
     path = normalize_path(path);
 
     if (opts.ackmate) {
@@ -148,9 +151,14 @@ void print_file_matches(const char *path, const char *buf, const size_t buf_len,
                     /* print headers for ackmate to parse */
                     print_line_number(line, ';');
                     for (; last_printed_match < cur_match; last_printed_match++) {
-                        fprintf(out_fd, "%lu %lu",
-                                (unsigned long)(matches[last_printed_match].start - prev_line_offset),
-                                (unsigned long)(matches[last_printed_match].end - matches[last_printed_match].start));
+                        /* Don't print negative offsets. This isn't quite right, but not many people use --ackmate */
+                        long start = (long)(matches[last_printed_match].start - prev_line_offset);
+                        if (start < 0) {
+                            start = 0;
+                        }
+                        fprintf(out_fd, "%li %li",
+                                start,
+                                (long)(matches[last_printed_match].end - matches[last_printed_match].start));
                         last_printed_match == cur_match - 1 ? fputc(':', out_fd) : fputc(',', out_fd);
                     }
                     print_line(buf, i, prev_line_offset);
@@ -259,9 +267,11 @@ void print_line_number(size_t line, const char sep) {
 
 void print_column_number(const match_t matches[], size_t last_printed_match,
                          size_t prev_line_offset, const char sep) {
-    fprintf(out_fd, "%lu%c",
-            (unsigned long)(matches[last_printed_match].start - prev_line_offset) + 1,
-            sep);
+    size_t column = 0;
+    if (prev_line_offset <= matches[last_printed_match].start) {
+        column = (matches[last_printed_match].start - prev_line_offset) + 1;
+    }
+    fprintf(out_fd, "%lu%c", (unsigned long)column, sep);
 }
 
 void print_file_separator(void) {
