@@ -36,7 +36,7 @@ void search_buf(const char *buf, const size_t buf_len,
         matches_spare = 0;
     }
 
-    if (opts.query_len == 1 && opts.query[0] == '.') {
+    if (!opts.literal && opts.query_len == 1 && opts.query[0] == '.') {
         matches_size = 1;
         matches = ag_malloc(matches_size * sizeof(match_t));
         matches[0].start = 0;
@@ -72,11 +72,7 @@ void search_buf(const char *buf, const size_t buf_len,
                 }
             }
 
-            if (matches_len + matches_spare >= matches_size) {
-                /* TODO: benchmark initial size of matches. 100 may be too small/big */
-                matches_size = matches ? matches_size * 2 : 100;
-                matches = ag_realloc(matches, matches_size * sizeof(match_t));
-            }
+            realloc_matches(&matches, &matches_size, matches_len + matches_spare);
 
             matches[matches_len].start = match_ptr - buf;
             matches[matches_len].end = matches[matches_len].start + opts.query_len;
@@ -92,28 +88,66 @@ void search_buf(const char *buf, const size_t buf_len,
         }
     } else {
         int offset_vector[3];
-        while (buf_offset < buf_len &&
-               (pcre_exec(opts.re, opts.re_extra, buf, buf_len, buf_offset, 0, offset_vector, 3)) >= 0) {
-            log_debug("Regex match found. File %s, offset %i bytes.", dir_full_path, offset_vector[0]);
-            buf_offset = offset_vector[1];
+        if (opts.multiline) {
+            while (buf_offset < buf_len &&
+                   (pcre_exec(opts.re, opts.re_extra, buf, buf_len, buf_offset, 0, offset_vector, 3)) >= 0) {
+                log_debug("Regex match found. File %s, offset %i bytes.", dir_full_path, offset_vector[0]);
+                buf_offset = offset_vector[1];
+                if (offset_vector[0] == offset_vector[1]) {
+                    ++buf_offset;
+                    log_debug("Regex match is of length zero. Advancing offset one byte.");
+                }
 
-            /* TODO: copy-pasted from above. FIXME */
-            if (matches_len + matches_spare >= matches_size) {
-                matches_size = matches ? matches_size * 2 : 100;
-                matches = ag_realloc(matches, matches_size * sizeof(match_t));
+                realloc_matches(&matches, &matches_size, matches_len + matches_spare);
+
+                matches[matches_len].start = offset_vector[0];
+                matches[matches_len].end = offset_vector[1];
+                matches_len++;
+
+                if (opts.max_matches_per_file > 0 && matches_len >= opts.max_matches_per_file) {
+                    log_err("Too many matches in %s. Skipping the rest of this file.", dir_full_path);
+                    break;
+                }
             }
+        } else {
+            while (buf_offset < buf_len) {
+                const char *line;
+                size_t line_len = buf_getline(&line, buf, buf_len, buf_offset);
+                if (!line) {
+                    break;
+                }
+                size_t line_offset = 0;
+                while (line_offset < line_len) {
+                    int rv = pcre_exec(opts.re, opts.re_extra, line, line_len, line_offset, 0, offset_vector, 3);
+                    if (rv < 0) {
+                        break;
+                    }
+                    size_t line_to_buf = buf_offset + line_offset;
+                    log_debug("Regex match found. File %s, offset %i bytes.", dir_full_path, offset_vector[0]);
+                    line_offset = offset_vector[1];
+                    if (offset_vector[0] == offset_vector[1]) {
+                        ++line_offset;
+                        log_debug("Regex match is of length zero. Advancing offset one byte.");
+                    }
 
-            matches[matches_len].start = offset_vector[0];
-            matches[matches_len].end = offset_vector[1];
-            matches_len++;
+                    realloc_matches(&matches, &matches_size, matches_len + matches_spare);
 
-            if (opts.max_matches_per_file > 0 && matches_len >= opts.max_matches_per_file) {
-                log_err("Too many matches in %s. Skipping the rest of this file.", dir_full_path);
-                break;
+                    matches[matches_len].start = offset_vector[0] + line_to_buf;
+                    matches[matches_len].end = offset_vector[1] + line_to_buf;
+                    matches_len++;
+
+                    if (opts.max_matches_per_file > 0 && matches_len >= opts.max_matches_per_file) {
+                        log_err("Too many matches in %s. Skipping the rest of this file.", dir_full_path);
+                        goto multiline_done;
+                    }
+                }
+                buf_offset += line_len + 1;
             }
         }
     }
 
+multiline_done:
+
     if (opts.invert_match) {
         matches_len = invert_matches(buf, buf_len, matches, matches_len);
     }
@@ -123,6 +157,9 @@ void search_buf(const char *buf, const size_t buf_len,
         stats.total_bytes += buf_len;
         stats.total_files++;
         stats.total_matches += matches_len;
+        if (matches_len > 0) {
+            stats.total_file_matches++;
+        }
         pthread_mutex_unlock(&stats_mtx);
     }
 
@@ -185,7 +222,7 @@ void search_file(const char *file_full_path) {
     char *buf = NULL;
     struct stat statbuf;
     int rv = 0;
-    FILE *pipe = NULL;
+    FILE *fp = NULL;
 
     fd = open(file_full_path, O_RDONLY);
     if (fd < 0) {
@@ -212,9 +249,9 @@ void search_file(const char *file_full_path) {
 
     if (statbuf.st_mode & S_IFIFO) {
         log_debug("%s is a named pipe. stream searching", file_full_path);
-        pipe = fdopen(fd, "r");
-        search_stream(pipe, file_full_path);
-        fclose(pipe);
+        fp = fdopen(fd, "r");
+        search_stream(fp, file_full_path);
+        fclose(fp);
         goto cleanup;
     }
 
@@ -378,7 +415,8 @@ static int check_symloop_leave(dirkey_t *dirkey) {
 /* TODO: Append matches to some data structure instead of just printing them out.
  * Then ag can have sweet summaries of matches/files scanned/time/etc.
  */
-void search_dir(ignores *ig, const char *base_path, const char *path, const int depth) {
+void search_dir(ignores *ig, const char *base_path, const char *path, const int depth,
+                dev_t original_dev) {
     struct dirent **dir_list = NULL;
     struct dirent *dir = NULL;
     scandir_baton_t scandir_baton;
@@ -429,6 +467,10 @@ void search_dir(ignores *ig, const char *base_path, const char *path, const int
                 if (opts.print_path == PATH_PRINT_DEFAULT || opts.print_path == PATH_PRINT_DEFAULT_EACH_LINE) {
                     opts.print_path = PATH_PRINT_NOTHING;
                 }
+                /* If we're only searching one file and --only-matching is specified, disable line numbers too. */
+                if (opts.only_matching && opts.print_path == PATH_PRINT_NOTHING) {
+                    opts.print_line_numbers = FALSE;
+                }
             }
             search_file(path);
         } else {
@@ -445,6 +487,19 @@ void search_dir(ignores *ig, const char *base_path, const char *path, const int
         queue_item = NULL;
         dir = dir_list[i];
         ag_asprintf(&dir_full_path, "%s/%s", path, dir->d_name);
+#ifndef _WIN32
+        if (opts.one_dev) {
+            struct stat s;
+            if (lstat(dir_full_path, &s) != 0) {
+                log_err("Failed to get device information for %s. Skipping...", dir->d_name);
+                goto cleanup;
+            }
+            if (s.st_dev != original_dev) {
+                log_debug("File %s crosses a device boundary (is probably a mount point.) Skipping...", dir->d_name);
+                goto cleanup;
+            }
+        }
+#endif
 
         /* If a link points to a directory then we need to treat it as a directory. */
         if (!opts.follow_symlinks && is_symlink(path, dir)) {
@@ -486,12 +541,13 @@ void search_dir(ignores *ig, const char *base_path, const char *path, const int
             if (depth < opts.max_search_depth || opts.max_search_depth == -1) {
                 log_debug("Searching dir %s", dir_full_path);
                 ignores *child_ig;
-                // #if defined(__MINGW32__) || defined(__CYGWIN__)
+#ifdef HAVE_DIRENT_DNAMLEN
+                child_ig = init_ignore(ig, dir->d_name, dir->d_namlen);
+#else
                 child_ig = init_ignore(ig, dir->d_name, strlen(dir->d_name));
-                // #else
-                // child_ig = init_ignore(ig, dir->d_name, dir->d_namlen);
-                // #endif
-                search_dir(child_ig, base_path, dir_full_path, depth + 1);
+#endif
+                search_dir(child_ig, base_path, dir_full_path, depth + 1,
+                           original_dev);
                 cleanup_ignore(child_ig);
             } else {
                 if (opts.max_search_depth == DEFAULT_MAX_SEARCH_DEPTH) {
@@ -499,7 +555,7 @@ void search_dir(ignores *ig, const char *base_path, const char *path, const int
                      * If the user didn't intentionally specify a particular depth,
                      * this is a warning...
                      */
-                    log_warn("Skipping %s. Use the --depth option to search deeper.", dir_full_path);
+                    log_err("Skipping %s. Use the --depth option to search deeper.", dir_full_path);
                 } else {
                     /* ... if they did, let's settle for debug. */
                     log_debug("Skipping %s. Use the --depth option to search deeper.", dir_full_path);
