@@ -52,6 +52,17 @@ char *ag_strndup(const char *s, size_t size) {
 #endif
 }
 
+void free_strings(char **strs, const size_t strs_len) {
+    if (strs == NULL) {
+        return;
+    }
+    size_t i;
+    for (i = 0; i < strs_len; i++) {
+        free(strs[i]);
+    }
+    free(strs);
+}
+
 void generate_alpha_skip(const char *find, size_t f_len, size_t skip_lookup[], const int case_sensitive) {
     size_t i;
 
@@ -247,13 +258,24 @@ size_t invert_matches(const char *buf, const size_t buf_len, match_t matches[],
     return inverted_match_count;
 }
 
+void realloc_matches(match_t **matches, size_t *matches_size, size_t matches_len) {
+    if (matches_len < *matches_size) {
+        return;
+    }
+    /* TODO: benchmark initial size of matches. 100 may be too small/big */
+    *matches_size = *matches ? *matches_size * 2 : 100;
+    *matches = ag_realloc(*matches, *matches_size * sizeof(match_t));
+}
+
 void compile_study(pcre **re, pcre_extra **re_extra, char *q, const int pcre_opts, const int study_opts) {
     const char *pcre_err = NULL;
     int pcre_err_offset = 0;
 
     *re = pcre_compile(q, pcre_opts, &pcre_err, &pcre_err_offset, NULL);
     if (*re == NULL) {
-        die("pcre_compile failed at position %i. Error: %s", pcre_err_offset, pcre_err);
+        die("Bad regex! pcre_compile() failed at position %i: %s\nIf you meant to search for a literal string, run ag with -Q",
+            pcre_err_offset,
+            pcre_err);
     }
     *re_extra = pcre_study(*re, study_opts, &pcre_err);
     if (*re_extra == NULL) {
@@ -277,6 +299,11 @@ int is_binary(const void *buf, const size_t buf_len) {
         return 0;
     }
 
+    if (buf_len >= 4 && strncmp(buf, "%PDF-", 5) == 0) {
+        /* PDF. This is binary. */
+        return 1;
+    }
+
     for (i = 0; i < total_bytes; i++) {
         if (buf_c[i] == '\0') {
             /* NULL char. It's binary */
@@ -538,13 +565,22 @@ ssize_t getline(char **lineptr, size_t *n, FILE *stream) {
 }
 #endif
 
+ssize_t buf_getline(const char **line, const char *buf, const size_t buf_len, const size_t buf_offset) {
+    const char *cur = buf + buf_offset;
+    ssize_t i;
+    for (i = 0; cur[i] != '\n' && (buf_offset + i < buf_len); i++) {
+    }
+    *line = cur;
+    return i;
+}
+
 #ifndef HAVE_REALPATH
 /*
  * realpath() for Windows. Turns slashes into backslashes and calls _fullpath
  */
 char *realpath(const char *path, char *resolved_path) {
     char *p;
-    char tmp[MAX_PATH + 1];
+    char tmp[_MAX_PATH + 1];
     strlcpy(tmp, path, sizeof(tmp));
     p = tmp;
     while (*p) {
