@@ -37,6 +37,7 @@ typedef struct {
     long total_bytes;
     long total_files;
     long total_matches;
+    long total_file_matches;
     struct timeval time_start;
     struct timeval time_end;
 } ag_stats;
@@ -70,6 +71,7 @@ const char *boyer_moore_strncasestr(const char *s, const char *find, const size_
 strncmp_fp get_strstr(enum case_behavior opts);
 
 size_t invert_matches(const char *buf, const size_t buf_len, match_t matches[], size_t matches_len);
+void realloc_matches(match_t **matches, size_t *matches_size, size_t matches_len);
 void compile_study(pcre **re, pcre_extra **re_extra, char *q, const int pcre_opts, const int study_opts);
 
 void *decompress(const ag_compression_type zip_type, const void *buf, const int buf_len, const char *dir_full_path, int *new_buf_len);
@@ -93,6 +95,8 @@ void die(const char *fmt, ...);
 
 void ag_asprintf(char **ret, const char *fmt, ...);
 
+ssize_t buf_getline(const char **line, const char *buf, const size_t buf_len, const size_t buf_offset);
+
 #ifndef HAVE_FGETLN
 char *fgetln(FILE *fp, size_t *lenp);
 #endif
